/*------------------------------------------------------------------------------
  Copyright (c) 2008 Ensolis, LLC. All Rights Reserved.
  ----------------------------------------------------------------------------*/
  
var FFPreviewTooltip = {
  CANVAS_WIDTH: 430,
  CANVAS_HEIGHT: 320,
  PAGE_WIDTH: 660,
  PAGE_HEIGHT: 500,
  PAGE_OFFSET_X: 0,
  PAGE_OFFSET_Y: 244,
   
  isLoading: false,
  tooltipNode: null,
  
  get loadDeck() { return document.getElementById("ff-load-deck"); },
  get loadImage() { return document.getElementById("ff-load-image"); },
  get previewImage() { return document.getElementById("ff-preview-image"); },
  get renderFrame() { return document.getElementById("ff-render-frame"); },
  get bundle() { return document.getElementById("ff-bundle");  },
  
  onLoad: function ffpt_onLoad() {
    var self = this;  
    renderFrame = this.renderFrame;
    renderFrame.addEventListener("load", function() { self.renderWindow(); }, true);
  },
      
  fill: function ffpt_fill(node) {
  
    //do not show a tooltip if disabled
    var enabled = node.getAttribute("tooltipenabled");
    enabled = (enabled == "true");
    if (!enabled)
      return false;

    // cancel the load if needed
    if (this.isLoading && (this.tooltipNode != node))
      this.cancel();
    
    // return early if we are already loading the node
    if (this.isLoading && (this.tooltipNode == node))
      return true;
    
    // see if we have rendered the node in the past
    this.tooltipNode = node;
    var preview = node.getAttribute("tooltippreview");
    if (preview) {
      this.isLoading = false;
      this.previewImage.src = preview;
      this.displayPreview();
      
    // start the rendering
    } else {
      this.isLoading = true;
      var renderFrame = this.renderFrame;
      renderFrame.setAttribute("src", this.buildURL(node));
      this.displayLoad();
    }  
    return true;
  },
  
  cancel: function ffpt_cancel() {
    const stopFlags = Components.interfaces.nsIWebNavigation.STOP_ALL;  
    var renderFrame = this.renderFrame;
    renderFrame.webNavigation.stop(stopFlags);
    this.tooltipNode = null;
    this.isLoading = false;
  },
  
  displayLoad: function ffpt_displayLoad() {
    this.loadImage.setAttribute("loading", "true");
    this.loadDeck.selectedIndex = 0;
  },
  
  displayPreview: function ffpt_displayPreview() {
    this.loadImage.removeAttribute("loading");
    this.loadDeck.selectedIndex = 1;
  },
  
  buildURL: function ffpt_buildURL(node) {
    var manager = Components.classes["@ensolis.com/forecastfox/manager-service;1"].getService(Components.interfaces.ffIManagerService);
    var url = node.getAttribute("url");
    if (url == "")
      url = this.bundle.getString("ff.url.default");
    
    var separator = (url.indexOf("?") != -1) ? "&" : "?";
    url += separator + "metric=" + manager.getPref("units.current");
    return url;
  },
  
  renderWindow: function ffpt_renderWindow() {
    
    // get a reference to the window object for the canvas
    var remoteWindow = this.renderFrame.contentWindow;

    // set the size of the canvas
    var w = this.PAGE_WIDTH;
    var h = this.PAGE_HEIGHT;
    var x = this.PAGE_OFFSET_X;
    var y = this.PAGE_OFFSET_Y;
    
    var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
    canvas.width = this.CANVAS_WIDTH;
    canvas.height = this.CANVAS_HEIGHT;

    // scale the canvas and draw on it
    var ctx = canvas.getContext("2d");
    ctx.scale(canvas.width/w, canvas.height/h);
    ctx.drawWindow(remoteWindow, x, y, w, h, "rgb(0,0,0)");    
    
    // set the preview image and display it
    var preview = canvas.toDataURL();    
    this.isLoading = false;
    this.previewImage.src = preview;
    this.tooltipNode.setAttribute("tooltippreview", preview);
    this.displayPreview();      
  }
};